==> Biostrings_2.6.6.tar.gz: Biostrings/R/XInteger.R <==
### =========================================================================
### External integer vectors: the "XInteger" class
### -------------------------------------------------------------------------
###
### The "XInteger" class implements the concept of "XRaw" objects but
### for integers instead of bytes.
### Some differences between "integer" and "XInteger":
###   1. an "XInteger" object can't be of length 0 (XInteger(0) produces an error)
###   2. XInteger(10) does not initialize its values (integer(10) does)
###   3. XInteger(10)[i] produces an error if i is out of bounds
###   4. XInteger objects are fast:
###        > a <- integer(100000000)
###        > system.time(tmp <- a[])
###        [1] 0.65 0.30 0.95 0.00 0.00
###        > system.time(a[] <- 100:1)
###        [1] 3.08 0.52 3.61 0.00 0.00
###
###        > ib <- XInteger(100000000)
###        > system.time(tmp <- ib[])
###        [1] 0.39 0.52 0.91 0.00 0.00
###        > system.time(ib[] <- 100:1)
###        [1] 0.56 0.00 0.56 0.00 0.00

setClass("XInteger", representation(xp="externalptr"))


### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Initialization

setMethod("initialize", "XInteger",
    function(.Object, length)
    {
        if (missing(length))
            stop("argument 'length' is missing")
        if (!is.numeric(length) || is.nan(length))
            stop("'length' is not a number")
        if (length(length) != 1)
            stop("'length' must have only one element")
        length <- as.integer(length)
        if (length < 1)
            stop("XInteger length must be >= 1")
        xp <- .Call("Biostrings_xp_new", PACKAGE="Biostrings")
        .Call("XInteger_alloc", xp, length, PACKAGE="Biostrings")
        .Object@xp <- xp
        .Object
    }
)

XInteger <- function(...)
{
    new("XInteger", ...)
}

setMethod("show", "XInteger",
    function(object)
    {
        .Call("XInteger_show", object@xp, PACKAGE="Biostrings")
    }
)

setMethod("length", "XInteger",
    function(x)
    {
        .Call("XInteger_length", x@xp, PACKAGE="Biostrings")
    }
)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Read/write functions.
### These are safe wrappers to unsafe C functions.
### If length(i) == 0 then the read functions return an empty vector
### and the write functions don't do anything.

XInteger.read <- function(x, i, imax=integer(0))
{
    if (!is.integer(i))
        i <- as.integer(i)
    if (length(i) == 1) {
        if (length(imax) == 0)
            imax <- i
        else
            imax <- as.integer(imax)
        .Call("XInteger_read_ints_from_i1i2", x@xp, i, imax, PACKAGE="Biostrings")
    } else {
        .Call("XInteger_read_ints_from_subset", x@xp, i, PACKAGE="Biostrings")
    }
}

XInteger.write <- function(x, i, imax=integer(0), value)
{
    if (!is.integer(value))
        stop("'value' is not an integer vector")
    if (!is.integer(i))
        i <- as.integer(i)
    if (length(i) == 1) {
        if (length(imax) == 0)
            imax <- i
        else
            imax <- as.integer(imax)
        .Call("XInteger_write_ints_to_i1i2", x@xp, i, imax, value, PACKAGE="Biostrings")
    } else {
        .Call("XInteger_write_ints_to_subset", x@xp, i, value, PACKAGE="Biostrings")
    }
    x
}


### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### length(as.integer(ib)) is equivalent to length(ib)
### but the latter is MUCH faster!
setMethod("as.integer", "XInteger",
    function(x)
    {
        XInteger.read(x, 1, length(x))
    }
)


### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Subsetting

setMethod("[", "XInteger",
    function(x, i, j, ..., drop)
    {
        if (!missing(j) || length(list(...)) > 0)
            stop("invalid subsetting")
        if (missing(i))
            return(as.integer(x))
        XInteger.read(x, i)
    }
)

setReplaceMethod("[", "XInteger",
    function(x, i, j,..., value)
    {
        if (!missing(j) || length(list(...)) > 0)
            stop("invalid subsetting")

        ## We want to allow this: ib[3] <- 4, even if storage.mode(value)
        ## is not "integer"
        if (!is.integer(value)) {
            if (length(value) >= 2)
                stop("'storage.mode(value)' must be \"integer\"")
            tmp <- value
            value <- as.integer(value)
            if (value != tmp)
                stop("'value' is not an integer")
        }
        ## Now 'value' is an integer vector
        if (missing(i))
            return(XInteger.write(x, 1, length(x), value=value))
        XInteger.write(x, i, value=value)
    }
)


### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Equality

setMethod("==", signature(e1="XInteger", e2="XInteger"),
    function(e1, e2)
    {
        address(e1@xp) == address(e2@xp)
    }
)
setMethod("!=", signature(e1="XInteger", e2="XInteger"),
    function(e1, e2)
    {
        address(e1@xp) != address(e2@xp)
    }
)

### A wrapper to the very fast memcmp() C-function.
### Arguments MUST be the following or it will crash R:
###   x1, x2: "XInteger" objects
###   start1, start2, width: single integers
### In addition: 1 <= start1 <= start1+width-1 <= length(x1)
###              1 <= start2 <= start2+width-1 <= length(x2)
### WARNING: This function is voluntarly unsafe (it doesn't check its
### arguments) because we want it to be the fastest possible!
XInteger.compare <- function(x1, start1, x2, start2, width)
{
    .Call("XInteger_memcmp", x1@xp, start1, x2@xp, start2, width, PACKAGE="Biostrings")
}

==> Biostrings_2.8.18.tar.gz: Biostrings/R/XInteger.R <==
### =========================================================================
### External integer vectors: the "XInteger" class
### -------------------------------------------------------------------------
###
### The "XInteger" class implements the concept of "XRaw" objects but
### for integers instead of bytes.
### Some differences between "integer" and "XInteger":
###
###   1. XInteger(10) does not initialize its values (integer(10) does)
###
###   2. XInteger(10)[i] produces an error if i is out of bounds
###
###   3. XInteger objects are faster:
###
###        > a <- integer(100000000)
###        > system.time(tmp <- a[])
###        [1] 0.65 0.30 0.95 0.00 0.00
###        > system.time(a[] <- 100:1)
###        [1] 3.08 0.52 3.61 0.00 0.00
###
###        > ib <- XInteger(100000000)
###        > system.time(tmp <- ib[])
###        [1] 0.39 0.52 0.91 0.00 0.00
###        > system.time(ib[] <- 100:1)
###        [1] 0.56 0.00 0.56 0.00 0.00

setClass("XInteger", representation(xp="externalptr"))


### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Initialization.
###
### Note that unlike integer vectors, XInteger objects are not initialized
### with 0's.
###

### This:
###   xi <- XInteger(30)
### will call this "initialize" method.
setMethod("initialize", "XInteger",
    function(.Object, length, verbose=FALSE)
    {
        if (!isSingleNumber(length))
            stop("'length' must be a single integer")
        length <- as.integer(length)
        if (length < 0)
            stop("'length' must be a non-negative integer")
        xp <- .Call("Biostrings_xp_new", PACKAGE="Biostrings")
        if (verbose)
            cat("Allocating memory for new", class(.Object), "object...")
        .Call("XInteger_alloc", xp, length, PACKAGE="Biostrings")
        if (verbose) {
            cat(" OK\n")
            show_string <- .Call("XInteger_get_show_string", xp, PACKAGE="Biostrings")
            cat("New", show_string, "successfully created\n")
        }
        .Object@xp <- xp
        .Object
    }
)

XInteger <- function(...)
{
    new("XInteger", ...)
}

setMethod("show", "XInteger",
    function(object)
    {
        show_string <- .Call("XInteger_get_show_string", object@xp, PACKAGE="Biostrings")
        cat(show_string, "\n", sep="")
        ## What is correct here? The documentation (?show) says that 'show'
        ## should return an invisible 'NULL' but, on the other hand, the 'show'
        ## method for intergers returns its 'object' argument...
        invisible(object)
    }
)

setMethod("length", "XInteger",
    function(x)
    {
        .Call("XInteger_length", x@xp, PACKAGE="Biostrings")
    }
)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Read/write functions.
### These are almost safe wrappers to unsafe C functions ("almost" because
### they don't check for NAs in their arguments).
### If length(i) == 0 then the read functions return an empty vector
### and the write functions don't do anything.

XInteger.read <- function(x, i, imax=integer(0))
{
    if (!is.integer(i))
        i <- as.integer(i)
    if (length(i) == 1) {
        if (length(imax) == 0)
            imax <- i
        else
            imax <- as.integer(imax)
        .Call("XInteger_read_ints_from_i1i2", x@xp, i, imax, PACKAGE="Biostrings")
    } else {
        .Call("XInteger_read_ints_from_subset", x@xp, i, PACKAGE="Biostrings")
    }
}

XInteger.write <- function(x, i, imax=integer(0), value)
{
    if (!is.integer(value))
        stop("'value' is not an integer vector")
    if (!is.integer(i))
        i <- as.integer(i)
    if (length(i) == 1) {
        if (length(imax) == 0)
            imax <- i
        else
            imax <- as.integer(imax)
        .Call("XInteger_write_ints_to_i1i2", x@xp, i, imax, value, PACKAGE="Biostrings")
    } else {
        .Call("XInteger_write_ints_to_subset", x@xp, i, value, PACKAGE="Biostrings")
    }
    x
}


### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### length(as.integer(ib)) is equivalent to length(ib)
### but the latter is MUCH faster!
setMethod("as.integer", "XInteger",
    function(x)
    {
        XInteger.read(x, 1, length(x))
    }
)


### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Subsetting

setMethod("[", "XInteger",
    function(x, i, j, ..., drop)
    {
        if (!missing(j) || length(list(...)) > 0)
            stop("invalid subsetting")
        if (missing(i))
            return(as.integer(x))
        XInteger.read(x, i)
    }
)

setReplaceMethod("[", "XInteger",
    function(x, i, j,..., value)
    {
        if (!missing(j) || length(list(...)) > 0)
            stop("invalid subsetting")

        ## We want to allow this: ib[3] <- 4, even if storage.mode(value)
        ## is not "integer"
        if (!is.integer(value)) {
            if (length(value) >= 2)
                stop("'storage.mode(value)' must be \"integer\"")
            tmp <- value
            value <- as.integer(value)
            if (value != tmp)
                stop("'value' is not an integer")
        }
        ## Now 'value' is an integer vector
        if (missing(i))
            return(XInteger.write(x, 1, length(x), value=value))
        XInteger.write(x, i, value=value)
    }
)


### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Equality

setMethod("==", signature(e1="XInteger", e2="XInteger"),
    function(e1, e2)
    {
        address(e1@xp) == address(e2@xp)
    }
)
setMethod("!=", signature(e1="XInteger", e2="XInteger"),
    function(e1, e2)
    {
        address(e1@xp) != address(e2@xp)
    }
)

### A wrapper to the very fast memcmp() C-function.
### Arguments MUST be the following or it will crash R:
###   x1, x2: "XInteger" objects
###   start1, start2, width: single integers
### In addition: 1 <= start1 <= start1+width-1 <= length(x1)
###              1 <= start2 <= start2+width-1 <= length(x2)
### WARNING: This function is voluntarly unsafe (it doesn't check its
### arguments) because we want it to be the fastest possible!
XInteger.compare <- function(x1, start1, x2, start2, width)
{
    .Call("XInteger_memcmp", x1@xp, start1, x2@xp, start2, width, PACKAGE="Biostrings")
}
